# copyright 2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-trustedauth pyramid_cubiweb integration module

"""

import logging

from zope.interface import implementer
from pyramid.authentication import IAuthenticationPolicy

logger = logging.getLogger(__name__)

# we cannot really use the RemoteUserAuthenticationPolicy here since
# it does **not** return the result of the callback -- where we could
# have done the login => eid translation -- call as
# authenticated_userid (pyramid_cubicweb expects this method to return
# a CWUser eid).


@implementer(IAuthenticationPolicy)
class TrustedAuthenticationPolicy(object):

    def __init__(self, environ_key='HTTP_X_REMOTE_USER'):
        self.environ_key = environ_key

    def unauthenticated_userid(self, request):
        return None

    def authenticated_userid(self, request):
        login = request.environ.get(self.environ_key)
        if login is None:
            return
        repo = request.registry['cubicweb.repository']
        with repo.internal_cnx() as cnx:
            try:
                rset = cnx.execute('Any U WHERE U is CWUser, U login %(login)s',
                                   {'login': login})
                if rset:
                    assert len(rset) == 1
                    logger.debug('%s: authenticated %s (%s)', self.__class__.__name__, login, rset[0][0])
                    return rset[0][0]
            except Exception as exc:
                logger.debug('%s: authentication failure (%s)', self.__class__.__name__, exc)
        return None

    def remember(self, request, principal, **kw):
        """ A no-op. The ``REMOTE_USER`` does not provide a protocol for
        remembering the user. This will be application-specific and can
        be done somewhere else or in a subclass."""
        return ()

    def forget(self, request):
        """ A no-op. The ``REMOTE_USER`` does not provide a protocol for
        forgetting the user. This will be application-specific and can
        be done somewhere else or in a subclass."""
        return ()



import webtest

from pyramid_cubicweb.tests import PyramidCWTest
from cubicweb_trustedauth import includeme
from cubicweb.web.controller import Controller


class TestController(Controller):
    __regid__ = 'testauth'

    def publish(self, rset):
        if self._cw.user.login == self._cw.form.get('expected', 'admin'):
            return 'VALID'
        else:
            return 'INVALID'


class PyramidTrustedAuthRequestTC(PyramidCWTest):
    test_db_id = 'trustedauth'

    def includeme(self, config):
        includeme(config)

    def _assert_auth_failed(self, req, result):
        self.assertEqual('INVALID', result)

    def _assert_auth(self, req, result):
        self.assertEqual(200, req.status_int)
        self.assertEqual('VALID', result)

    def _test_header_format(self, hkey, login, http_method='GET',
                            headers=None, content=None, url='/testauth', **params):
        if headers is None:
            headers = {}
        headers[hkey] = login
        req = webtest.TestRequest.blank(url, base_url=self.config['base-url'].rstrip('/'),
                                        method=http_method,
                                        headers=headers, **params)
        if http_method == 'POST':
            if content is None:
                content = "rql=Any+X+WHERE+X+is+Player"
        if content:
            req.body = content
        with self.temporary_appobjects(TestController):
            resp = self.webapp.do_request(req)
        return resp.body, resp

    def test_login(self):
        result, req = self._test_header_format(hkey='X-Remote-User',
                                               login='admin',
                                               )
        self._assert_auth(req, result)

    def test_bad_login(self):
        result, req = self._test_header_format(hkey='X-Remote-User',
                                               login='admin2',
                                               )
        self._assert_auth_failed(req, result)

    def test_bad_key(self):
        result, req = self._test_header_format(hkey='X-Remote-User2',
                                               login='admin',
                                               )
        self._assert_auth_failed(req, result)


if __name__ == "__main__":
    from logilab.common.testlib import unittest_main
    unittest_main()



# copyright 2010-2015 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""cubicweb-trustedauth application package

authentication plugin for cubicweb instances behind a trusted
reverse proxy managing authentication  (eg. apache + kerberos)
"""


def includeme(config):
    """Activate the trusted authentication policy in a
    pyramid_cubiweb instance.

    Usually called from the main pyramid_cubicweb.core configurator
    (see ``pyramid_cubicweb.core``).

    See also :ref:`defaults_module`

    """
    from cubicweb_trustedauth.pconfig import TrustedAuthenticationPolicy
    policy = TrustedAuthenticationPolicy()

    # # add some bw compat methods
    # # these ease code factorization between pyramid related code and legacy one
    # config.add_request_method(
    #     lambda req, header, default=None: req.headers.get(header, default),
    #     name='get_header', property=False, reify=False)
    # config.add_request_method(lambda req: req.method,
    #     name='http_method', property=False, reify=False)

    if config.registry.get('cubicweb.authpolicy') is None:
        err = "trustedauth: the default cubicweb auth policy should be "\
              "available via the 'cubicweb.authpolicy' registry config "\
              "entry"
        raise ValueError(err)

    # if we use (the default) a multiauth policy in CW, append
    # signedrequest to it
    mainpolicy = config.registry['cubicweb.authpolicy']
    mainpolicy._policies.append(policy)
